# Copyright (c) 2024 Microsoft Corporation.
# Licensed under the MIT License

"""Default values for the fine-tuning module."""

DEFAULT_TASK = """
Identify the relations and structure of the community of interest, specifically within the {domain} domain.
"""

MAX_TOKEN_COUNT = 2000
