# Copyright (c) 2024 Microsoft Corporation.
# Licensed under the MIT License

"""A file containing TRANSLATION_PROMPT value definition."""

TRANSLATION_PROMPT = """
    You are a helpful assistant. Translate into {language} the following text, and make sure all of the text is in {language}.
    """.strip()
