# Copyright (c) 2024 Microsoft Corporation.
# Licensed under the MIT License

"""The Indexing Engine text translate package root."""

from .text_translate import text_translate

__all__ = ["text_translate"]
