# Copyright (c) 2024 Microsoft Corporation.
# Licensed under the MIT License

"""The Indexing Engine text extract claims package root."""

from .extract_covariates import ExtractClaimsStrategyType, extract_covariates

__all__ = ["ExtractClaimsStrategyType", "extract_covariates"]
