# Copyright (c) 2024 Microsoft Corporation.
# Licensed under the MIT License

"""The Indexing Engine text embed package root."""

from .text_embed import TextEmbedStrategyType, text_embed

__all__ = ["TextEmbedStrategyType", "text_embed"]
