# Copyright (c) 2024 Microsoft Corporation.
# Licensed under the MIT License

"""Root package for resolution entities."""

from .description_summarize import SummarizeStrategyType, summarize_descriptions

__all__ = ["SummarizeStrategyType", "summarize_descriptions"]
