# Copyright (c) 2024 Microsoft Corporation.
# Licensed under the MIT License

"""Indexing Engine - Summarization Strategies Package."""

from .typing import SummarizationStrategy

__all__ = ["SummarizationStrategy"]
