# Copyright (c) 2024 Microsoft Corporation.
# Licensed under the MIT License

"""The Indexing Engine graph intelligence package root."""

from .run_graph_intelligence import run_gi

__all__ = ["run_gi"]
