# Copyright (c) 2024 Microsoft Corporation.
# Licensed under the MIT License

"""The Indexing Engine text chunk package root."""

from .text_chunk import ChunkStrategy, ChunkStrategyType, chunk

__all__ = ["ChunkStrategy", "ChunkStrategyType", "chunk"]
