# Copyright (c) 2024 Microsoft Corporation.
# Licensed under the MIT License

"""A file containing MOCK_LLM_RESPONSES definition."""

MOCK_LLM_RESPONSES = [
    """
[
    {
        "subject": "COMPANY A",
        "object": "GOVERNMENT AGENCY B",
        "type": "ANTI-COMPETITIVE PRACTICES",
        "status": "TRUE",
        "start_date": "2022-01-10T00:00:00",
        "end_date": "2022-01-10T00:00:00",
        "description": "Company A was found to engage in anti-competitive practices because it was fined for bid rigging in multiple public tenders published by Government Agency B according to an article published on 2022/01/10",
        "source_text": ["According to an article published on 2022/01/10, Company A was fined for bid rigging while participating in multiple public tenders published by Government Agency B."]
    }
]
    """.strip()
]
