# Copyright (c) 2024 Microsoft Corporation.
# Licensed under the MIT License

"""The Indexing Engine entities extraction package root."""

from .entity_extract import ExtractEntityStrategyType, entity_extract

__all__ = ["ExtractEntityStrategyType", "entity_extract"]
