# Copyright (c) 2024 Microsoft Corporation.
# Licensed under the MIT License

"""The Indexing Engine covariates package root."""

from .extract_covariates import extract_covariates

__all__ = ["extract_covariates"]
