# Copyright (c) 2024 Microsoft Corporation.
# Licensed under the MIT License

"""The Indexing Engine graph report strategies graph intelligence package root."""

from .run_graph_intelligence import run

__all__ = ["run"]
